//#Illustrator

/*
  MasterPager.jsx for Adobe Illustrator
  Description: numbering pages or artboards, displaying the file name, and incorporating the current date.
  Date: May, 2024
 
  Author: Katja Bjerrum, email: katja@productivista.com , www.productivista.com
  ============================================================================
  NOTICE:
  This script is provided "as is" without warranty of any kind.
  Free to use, not for sale.
  ============================================================================
  Released under the MIT license.
  http://opensource.org/licenses/mit-license.php
  ============================================================================
  Please contact me if you need some help or customization of this script
  at katja@productivista.com
*/

var doc = app.activeDocument;
app.coordinateSystem = CoordinateSystem.DOCUMENTCOORDINATESYSTEM;
var sel = app.activeDocument.selection;
if (layerExists('Master')) {
var lay = doc.layers.getByName('Master');


var ab = doc.artboards;
var dateArr = ['<wd>', '<dd>', '<mm>', '<yyyy>', ' <yy>', '<WD>', '<DD>', '<MM>', '<YYYY>', '<YY>'];
var nameArr = ['<name>', '<nameex>', '<NAME>', '<NAMEEX>'];
var pageArr = ['<p>', '<ps>', '<P>', '<PS>', '<pp>', '<PP>', '<Pp>', '<pP>', '<pps>', '<PPS>', '<pPs>', '<PPs>', '<Pps>', '<PpS>', '<pPS>'];


var languageMappings = {
    'LanguageType.ENGLISH': {
        'Month': ['January', 'February', 'March', 'April',
            'May', 'June', 'July', 'August',
            'September', 'October', 'November', 'December'],

        'WeekDay': ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday']
    },
    'LanguageType.RUSSIAN': {
        'Month': ['января', 'февраля', 'марта', 'апреля',
            'мая', 'июня', 'июля', 'августа',
            'сентября', 'октября', 'ноября', 'декабря'],
        'WeekDay': ['Воскресенье', 'Понедельник', 'Вторник', 'Среда', 'Четверг', 'Пятница', 'Суббота']
    },
    'LanguageType.BULGARIAN': {
        'Month': ['Януари', 'Февруари', 'Март', 'Април', 'Май', 'Юни', 'Юли', 'Август', 'Септември', 'Октомври', 'Ноември', 'Декември'],
        'WeekDay': ['Неделя', 'Понеделник', 'Вторник', 'Сряда', 'Четвъртък', 'Петък', 'Събота']
    },
    'LanguageType.CROATIAN': {
        'Month': ['siječnja', 'veljače', 'ožujka', 'travnja', 'svibnja', 'lipnja', 'srpnja', 'kolovoza', 'rujna', 'listopada', 'studenog', 'prosinca'],
        'WeekDay': ['Nedjelja', 'Ponedjeljak', 'Utorak', 'Srijeda', 'Četvrtak', 'Petak', 'Subota']
    },
    'LanguageType.CZECH': {
        'Month': ['Leden', 'Únor', 'Březen', 'Duben',
            'Květen', 'Červen', 'Červenec', 'Srpen',
            'Září', 'Říjen', 'Listopad', 'Prosinec'],
        'WeekDay': ['Neděle', 'Pondělí', 'Úterý', 'Středa', 'Čtvrtek', 'Pátek', 'Sobota']
    },
    'LanguageType.DANISH': {
        'Month': ['Januar', 'Februar', 'Marts', 'April',
            'Maj', 'Juni', 'Juli', 'August',
            'September', 'Oktober', 'November', 'December'],
        'WeekDay': ['Søndag', 'Mandag', 'Tirsdag', 'Onsdag', 'Torsdag', 'Fredag', 'Lørdag']
    },
    'LanguageType.DUTCH': {
        'Month': ['Januari', 'Februari', 'Maart', 'April',
            'Mei', 'Juni', 'Juli', 'Augustus',
            'September', 'Oktober', 'November', 'December'],
        'WeekDay': ['Zondag', 'Maandag', 'Dinsdag', 'Woensdag', 'Donderdag', 'Vrijdag', 'Zaterdag']
    },
    'LanguageType.ESTONIAN': {
        'Month': ['Jaanuar', 'Veebruar', 'Märts', 'Aprill',
            'Mai', 'Juuni', 'Juuli', 'August',
            'September', 'Oktoober', 'November', 'Detsember'],
        'WeekDay': ['Pühapäev', 'Esmaspäev', 'Teisipäev', 'Kolmapäev', 'Neljapäev', 'Reede', 'Laupäev']
    },
    'LanguageType.FINNISH': {
        'Month': ['Tammikuu', 'Helmikuu', 'Maaliskuu', 'Huhtikuu',
            'Toukokuu', 'Kesäkuu', 'Heinäkuu', 'Elokuu',
            'Syyskuu', 'Lokakuu', 'Marraskuu', 'Joulukuu'],
        'WeekDay': ['Sunnuntai', 'Maanantai', 'Tiistai', 'Keskiviikko', 'Torstai', 'Perjantai', 'Lauantai']
    },
    'LanguageType.FRENCH': {
        'Month': ['Janvier', 'Février', 'Mars', 'Avril',
            'Mai', 'Juin', 'Juillet', 'Août',
            'Septembre', 'Octobre', 'Novembre', 'Décembre'],
        'WeekDay': ['Dimanche', 'Lundi', 'Mardi', 'Mercredi', 'Jeudi', 'Vendredi', 'Samedi']
    },
    'LanguageType.GERMAN': {
        'Month': ['Januar', 'Februar', 'März', 'April',
            'Mai', 'Juni', 'Juli', 'August',
            'September', 'Oktober', 'November', 'Dezember'],
        'WeekDay': ['Sonntag', 'Montag', 'Dienstag', 'Mittwoch', 'Donnerstag', 'Freitag', 'Samstag']
    },
    'LanguageType.GREEK': {
        'Month': ['Ιανουάριος', 'Φεβρουάριος', 'Μάρτιος', 'Απρίλιος',
            'Μάιος', 'Ιούνιος', 'Ιούλιος', 'Αύγουστος',
            'Σεπτέμβριος', 'Οκτώβριος', 'Νοέμβριος', 'Δεκέμβριος'],
        'WeekDay': ['Κυριακή', 'Δευτέρα', 'Τρίτη', 'Τετάρτη', 'Πέμπτη', 'Παρασκευή', 'Σάββατο']
    },
    'LanguageType.ITALIAN': {
        'Month': ['Gennaio', 'Febbraio', 'Marzo', 'Aprile',
            'Maggio', 'Giugno', 'Luglio', 'Agosto',
            'Settembre', 'Ottobre', 'Novembre', 'Dicembre'],
        'WeekDay': ['Domenica', 'Lunedì', 'Martedì', 'Mercoledì', 'Giovedì', 'Venerdì', 'Sabato']
    },
    'LanguageType.LATVIAN': {
        'Month': ['Janvāris', 'Februāris', 'Marts', 'Aprīlis',
            'Maijs', 'Jūnijs', 'Jūlijs', 'Augusts',
            'Septembris', 'Oktobris', 'Novembris', 'Decembris'],
        'WeekDay': ['Svētdiena', 'Pirmdiena', 'Otrdiena', 'Trešdiena', 'Ceturtdiena', 'Piektdiena', 'Sestdiena']
    },
    'LanguageType.LITHUANIAN': {
        'Month': ['Sausis', 'Vasaris', 'Kovas', 'Balandis',
            'Gegužė', 'Birželis', 'Liepa', 'Rugpjūtis',
            'Rugsėjis', 'Spalis', 'Lapkritis', 'Gruodis'],
        'WeekDay': ['Sekmadienis', 'Pirmadienis', 'Antradienis', 'Trečiadienis', 'Ketvirtadienis', 'Penktadienis', 'Šeštadienis']
    },
    'LanguageType.NORWEGIAN': {
        'Month': ['Januar', 'Februar', 'Mars', 'April',
            'Mai', 'Juni', 'Juli', 'August',
            'September', 'Oktober', 'November', 'Desember'],
        'WeekDay': ['Søndag', 'Mandag', 'Tirsdag', 'Onsdag', 'Torsdag', 'Fredag', 'Lørdag']
    },
    'LanguageType.POLISH': {
        'Month': ['stycznia', 'lutego', 'marca', 'kwietnia', 'maja', 'czerwca', 'lipca', 'sierpnia', 'września', 'października', 'listopada', 'grudnia'],
        'WeekDay': ['Niedziela', 'Poniedziałek', 'Wtorek', 'Środa', 'Czwartek', 'Piątek', 'Sobota']
    },
    'LanguageType.PORTUGUESE': {
        'Month': ['Janeiro', 'Fevereiro', 'Março', 'Abril',
            'Maio', 'Junho', 'Julho', 'Agosto',
            'Setembro', 'Outubro', 'Novembro', 'Dezembro'],
        'WeekDay': ['Domingo', 'Segunda-feira', 'Terça-feira', 'Quarta-feira', 'Quinta-feira', 'Sexta-feira', 'Sábado']
    },
    'LanguageType.ROMANIAN': {
        'Month': ['Ianuarie', 'Februarie', 'Martie', 'Aprilie',
            'Mai', 'Iunie', 'Iulie', 'August',
            'Septembrie', 'Octombrie', 'Noiembrie', 'Decembrie'],
        'WeekDay': ['Duminică', 'Luni', 'Marți', 'Miercuri', 'Joi', 'Vineri', 'Sâmbătă']
    },
    'LanguageType.SLOVAK': {
        'Month': ['januára', 'februára', 'marca', 'apríla', 'mája', 'júna', 'júla', 'augusta', 'septembra', 'októbra', 'novembra', 'decembra'],
        'WeekDay': ['Nedeľa', 'Pondelok', 'Utorok', 'Streda', 'Štvrtok', 'Piatok', 'Sobota']
    },
    'LanguageType.SPANISH': {
        'Month': ['Enero', 'Febrero', 'Marzo', 'Abril',
            'Mayo', 'Junio', 'Julio', 'Agosto',
            'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'],
        'WeekDay': ['Domingo', 'Lunes', 'Martes', 'Miércoles', 'Jueves', 'Viernes', 'Sábado']
    },
    'LanguageType.SWEDISH': {
        'Month': ['Januari', 'Februari', 'Mars', 'April',
            'Maj', 'Juni', 'Juli', 'Augusti',
            'September', 'Oktober', 'November', 'December'],
        'WeekDay': ['Söndag', 'Måndag', 'Tisdag', 'Onsdag', 'Torsdag', 'Fredag', 'Lördag']
    },
    'LanguageType.UKRANIAN': {
        'Month': ['січня', 'лютого', 'березня', 'квітня', 'травня', 'червня', 'липня', 'серпня', 'вересня', 'жовтня', 'листопада', 'грудня'],
        'WeekDay': ['Неділя', 'Понеділок', 'Вівторок', 'Середа', 'Четвер', 'Пʼятниця', 'Субота']
    }
};



    var itemsDict = pushItems(lay.pageItems, dateArr, nameArr, pageArr);
    var TextDict = itemsDict[0];
    var GroupDict = itemsDict[1];
    
    for (var key in TextDict) {
    
        for (var j = 0; j < TextDict[key].length; j++) {
            if (key == 'Page Nr') {
                var artbordIndex = activateArtboardWithSelectedItem(TextDict[key][j]);
               
                displayPageNumbersOnSelectedArtboards(TextDict[key][j], artbordIndex) //displayPageNumbersTxtfromAB(TextDict[key][j]);
            }
                else if (key == 'Doc Name') {
                    displayDocNameTxtfromSelectedAB(TextDict[key][j]);
                }
                 else if (key == 'Date') {
                    displayDateTxtonSelectedAB(TextDict[key][j]);
                 }
                // else {
                //     alert('Can not find the right dictionary')
                // }
    
        }
    
    }
    
 
}
else{
    alert('Layer "Master" is missing. Please create a layer named "Master".');
}


function displayDateTxtonSelectedAB(myItem) {
 
    var doc = app.activeDocument;
    var myABindex = activateArtboardWithSelectedItem(myItem);
    var paragraph = myItem.textRange.paragraphs[0]; // Assuming there is at least one paragraph
    var language = paragraph.language; // Ensure this property exists and is correctly referenced
    var DateArr = getDate(language); // Ensure getDate function returns a well-formatted array

    var startPoint = getStartPoint(doc.artboards[myABindex], myItem); // Should start from the indexed artboard
    var myDir = getDirNew(myItem, startPoint, doc.artboards[myABindex] );
    
    var dayDig = DateArr[0] < 10 ? '0' + DateArr[0] : DateArr[0];
    var monthDig = DateArr[1] < 10 ? '0' + DateArr[1] : DateArr[1];
    var monthLat = ["I", "II", "III", "IV", "V", "VI", "VII", "VIII", "IX", "X", "XI", "XII"];
    var latDig = monthLat[DateArr[1] - 1]; // Direct access to correct month

    var inputString = myItem.contents;
    var content = inputString.replace(/<dd>/gi, dayDig)
                             .replace(/<d>/gi,  DateArr[0])
                             .replace(/<m>/gi, monthDig)
                             .replace(/<mm>/gi, DateArr[1])
                             .replace(/<lm>/gi, latDig)
                             .replace(/<yy>/gi, DateArr[4])
                             .replace(/<yyyy>/gi, DateArr[2])
                             .replace(/<month>/gi, DateArr[3])
                             .replace(/<wd>/gi, DateArr[5]);

   
        myItem.contents = content;
        myItem.name = myItem.name + (i + 1);
        var newCoord = getCoordsfromBorder(myItem, startPoint, doc.artboards[myABindex], myDir);
       
        myItem.position = newCoord;
 
};


function displayPageNumbersOnSelectedArtboards(myItem) {
    var doc = app.activeDocument;
    var myABindex = activateArtboardWithSelectedItem(myItem);
    var startPoint = getStartPoint(doc.artboards[myABindex], myItem); // Use the first specified artboard as the start point
    var myDir = getDirNew(myItem, startPoint, doc.artboards[myABindex] );
   
    var ABlength = doc.artboards.length;

        var currentPage = myABindex + 1; // Page number should start from 1 for the first specified artboard
        var pageDig = currentPage < 10 ? '0' + currentPage : currentPage;
        var pagesDig = ABlength < 10 ? '0' + (ABlength) : (ABlength );

        var inputString = myItem.contents;
        var newContent = inputString
            .replace(/<p>/g, currentPage)
            .replace(/<P>/g, currentPage)
            .replace(/<pp>/g, pageDig)
            .replace(/<pP>/g, pageDig)
            .replace(/<Pp>/g, pageDig)
            .replace(/<PP>/g, pageDig);
        var finalContent = newContent
            .replace(/<ps>/g, ABlength)
            .replace(/<PS>/g, ABlength )
            .replace(/<Ps>/g, ABlength )
            .replace(/<pS>/g, ABlength )
            .replace(/<pps>/g, pagesDig)
            .replace(/<pPS>/g, pagesDig)
            .replace(/<PPS>/g, pagesDig)
            .replace(/<PpS>/g, pagesDig)
            .replace(/<PPs>/g, pagesDig)
            .replace(/<Pps>/g, pagesDig)
            .replace(/<ppS>/g, pagesDig);
        var itext = myItem.duplicate();
        itext.contents = finalContent;
        itext.name = myItem.name + currentPage;

        var newCoord = getCoordsfromBorder(itext, startPoint, doc.artboards[myABindex], myDir);
        itext.position = newCoord;

    myItem.remove();
};


function displayDocNameTxtfromSelectedAB(myItem) {
    var doc = app.activeDocument;
    var myABindex = activateArtboardWithSelectedItem(myItem);
    var ABlength = doc.artboards.length;
    var startPoint = getStartPoint(doc.artboards[myABindex], myItem);
    var myDir = getDirNew(myItem, startPoint, doc.artboards[myABindex] );
   
    var textArr = [];
    var inputString = myItem.contents;
    var filenameEx = app.activeDocument.name;
    var fileName = filenameEx.split('.').slice(0, -1).join('.');
    var content = inputString.replace(/<Name>/g, '<name>').replace(/<NAME>/g, '<name>').replace(/<NAme>/g, '<name>').replace(/<nAme>/g, '<name>').replace(/<naMe>/g, '<name>');
    var finalContent = content.replace(/<name>/g, fileName).replace(/<nameex>/g, filenameEx);


    for (var i = myABindex; i < ABlength; i++) {
       
        myItem.contents = finalContent;
        myItem.name = myItem.name + (i + 1); // Append current artboard number to duplicated text item name
        textArr.push(myItem);
    }

    // Position each text item appropriately on its respective artboard
    for (var j = 0; j < textArr.length; j++) {
        var newCoord = getCoordsfromBorder(textArr[j], startPoint, doc.artboards[myABindex], myDir);
      
            textArr[j].position = newCoord;
       
       
    }

    
};



function ItemsNewPos(myArtboard, oldpos, new_coord) {
    var rect = myArtboard.artboardRect;
    var x_ori = rect[0];
    var y_ori = rect[1];
    var newPos = [oldpos[0] - x_ori + new_coord[0], oldpos[1] - y_ori + new_coord[1]];

    return newPos;

}

function getDate(Language) {
    var today = new Date();
    var date = today.getDate();
    var month = today.getMonth() + 1;
    // if (date < 10) { date = "0" + today.getDate() } else { date = today.getDate() };
    // if (month < 10) { month = "0" + (today.getMonth() + 1) } else { month = (today.getMonth() + 1) };
    var monthDays = languageMappings[Language];
    if (!monthDays) {
        monthDays = languageMappings[LanguageType.ENGLISH];
    }
    var monthNames = monthDays['Month'];
    var weekdayNames = monthDays['WeekDay'];

    var currentDay = new Date().getDay();
    var weekdayName = weekdayNames[currentDay];
    var currentMonth = new Date().getMonth();
    var monthName = monthNames[currentMonth];

    var fullYear = today.getFullYear();
    var lastTwoDigits = fullYear % 100;
    return [date, month, today.getFullYear(), monthName, lastTwoDigits, weekdayName];

};

function containsAnyPattern(textItem, patterns) {
    for (var i = 0; i < patterns.length; i++) {
        if (textItem.contents.indexOf(patterns[i]) !== -1) {
            return true;
        }
    }
    return false;
}


function pushItems(items, dateArr, nameArr, pageArr) {
    var itemsNamesDict = {
        'Page Nr': [],
        'Doc Name': [],
        'Date': []

    };

    var groupNamesDict = {
        'Page Nr': [],
        'Doc Name': [],
        'Date': []
    };
    for (var i = 0; i < items.length; i++) {

        if (items[i].typename == 'TextFrame') {
            var textFrame = items[i];

            if (containsAnyPattern(textFrame, pageArr)) {
                itemsNamesDict['Page Nr'].push(textFrame); // Add the text frame to the array if it matches
            } else if (containsAnyPattern(textFrame, nameArr)) {
                itemsNamesDict['Doc Name'].push(textFrame);
            } else if (containsAnyPattern(textFrame, dateArr)) {
                itemsNamesDict['Date'].push(textFrame);
            }
        }
        if (items[i].typename == 'GroupItem') {

            //if (items[i].typename == 'TextFrame') {
            var myGroup = items[i];


            for (var j = 0; j < myGroup.pageItems.length; j++) {
                var textFrame = myGroup.pageItems[j];
                if (textFrame.typename == 'TextFrame') {

                    if (textFrame.typename == 'TextFrame') {
                        if (containsAnyPattern(textFrame, pageArr)) {
                            if (!containsGroup(groupNamesDict['Page Nr'], myGroup)) {
                                groupNamesDict['Page Nr'].push(myGroup);
                            }
                        } else if (containsAnyPattern(textFrame, nameArr)) {
                            if (!containsGroup(groupNamesDict['Doc Name'], myGroup)) {
                                groupNamesDict['Doc Name'].push(myGroup);
                            }
                        } else if (containsAnyPattern(textFrame, dateArr)) {
                            if (!containsGroup(groupNamesDict['Date'], myGroup)) {
                                groupNamesDict['Date'].push(myGroup);
                            }
                        }
                    }



                }
            }


        }
    }

    return [itemsNamesDict, groupNamesDict];

}

function containsGroup(groupArray, group) {

    for (var i = 0; i < groupArray.length; i++) {
        var existingGroup = groupArray[i];
        if (existingGroup.name === group.name && existingGroup.position.toString() === group.position.toString()) {
            return true; // Found a match, no need to add
        }
    }
    return false; // No match found, safe to add
}



function checkContains(array, text) {
    for (var i = 0; i < array.length; i++) {
        if (text.indexOf(array[i]) >= 0) {
            return true;
        }
    }
    return false;
}

function capitalize(str) {
    //Capitalaze first letters
    return str.replace(/\b\w/g, function (l) {
        return l.toUpperCase();
    });
};


function find_items(myItem, collect_function) {
    function return_all(myItem, arr) {
        arr.push(myItem)
        return arr
    }
    if (typeof collect_function == 'undefined') {
        collect_function = return_all
    }

    var arr = [];

    return find_items_recursive(myItem, arr, collect_function)
};


function find_items_recursive(myItem, arr, collect_function) {

    if (myItem.typename == "GroupItem") {
        for (var i = 0; i < myItem.pageItems.length; i++) {
            var arr = find_items_recursive(myItem.pageItems[i], arr, collect_function);
        }
    }
    else if (myItem.typename == "PageItems") {
        for (var i = 0; i < myItem.length; i++) {
            var arr = find_items_recursive(myItem[i], arr, collect_function);
        }
    }
    else {
        var arr = collect_function(myItem, arr)
    }

    return arr;
};

//coords skal være defñ®†
function textInFrame(myItem, arr) {
    if (myItem.typename == "TextFrame") { // {

        arr.push(myItem);
    }
    return arr
};

function getDirNew(myItem, startPoint, artboard) {
    var bounds = myItem.geometricBounds;
    var centerX = (bounds[0] + bounds[2]) / 2;
    var centerY = (bounds[1] + bounds[3]) / 2;
    var artboardBounds = artboard.artboardRect;
    var artboardCenterX = (artboardBounds[0] + artboardBounds[2]) / 2;
    var artboardCenterY = (artboardBounds[1] + artboardBounds[3]) / 2;
    var tolerance = 1; // Adjust this value as needed to define "center"

   if (Math.round(centerX) != Math.round(artboardCenterX) && startPoint['coord'][0] == bounds[0] && startPoint['coord'][1] == bounds[1]) {
        return 'TL';
    } else if (Math.round(centerX) != Math.round(artboardCenterX) && startPoint['coord'][0] == bounds[0] && startPoint['coord'][1] == bounds[3]) {
        return 'BL';
    } else if (Math.round(centerX) != Math.round(artboardCenterX) && startPoint['coord'][0] == bounds[2] && startPoint['coord'][1] == bounds[1]) {
        return 'TR';
    } else if (Math.round(centerX) != Math.round(artboardCenterX) && startPoint['coord'][0] == bounds[2] && startPoint['coord'][1] == bounds[3]) {
        return 'BR';
    }

    if (Math.round(centerX) == Math.round(artboardCenterX)  && startPoint['coord'][1] == bounds[1]){
        return 'CenterT';
    } else if (Math.abs((centerX - artboardCenterX) < tolerance) && startPoint['coord'][1] == bounds[3]) {
        return 'CenterB';
   
    }
}

function getCoordsfromBorder(myItem, startPoint, AB, dir) {

    var ABrect = AB.artboardRect;
    var startCoord;

    if (dir == 'TL') {
        startCoord = [ABrect[0] + startPoint['dist'][0], ABrect[1] - startPoint['dist'][1]];
    } else if (dir == 'BL') {
        startCoord = [ABrect[0] + startPoint['dist'][0], ABrect[3] + startPoint['dist'][1] + myItem.height];
    } else if (dir == 'TR') {
        startCoord = [ABrect[2] - startPoint['dist'][0] - myItem.width, ABrect[1] - startPoint['dist'][1]];
    } else if (dir == 'BR') {
        startCoord = [ABrect[2] - startPoint['dist'][0] - myItem.width, ABrect[3] + startPoint['dist'][1] + myItem.height];
    }
    else if (dir == 'CenterT'){
        startCoord = [((ABrect[0] + ABrect[2])- myItem.width)/ 2 , ABrect[1] - startPoint['dist'][1]];
    }
    else if (dir == 'CenterB'){
        startCoord = [((ABrect[0] + ABrect[2])- myItem.width)/ 2 , ABrect[3] + startPoint['dist'][1] + myItem.height];
    }

    return startCoord;
}

function getCoordsfromBorderGroup(myItem, startPoint, AB, dir) {

    var ABrect = AB.artboardRect;
    var startCoord;

    if (dir == 'TL') {
        startCoord = [ABrect[0] + startPoint['dist'][0], ABrect[1] - startPoint['dist'][1]];
    } else if (dir == 'BL') {
        startCoord = [ABrect[0] + startPoint['dist'][0], ABrect[3] + startPoint['dist'][1] + myItem.height];
    } else if (dir == 'TR') {
        startCoord = [ABrect[2] - startPoint['dist'][0] - myItem.width, ABrect[1] - startPoint['dist'][1]];
    } else if (dir == 'BR') {
        startCoord = [ABrect[2] - startPoint['dist'][0] - myItem.width, ABrect[3] + startPoint['dist'][1] + myItem.height];
    }
   

    return startCoord;
}

function minstDist(coord1, arr) {
    //Finds min distance from point and array, push points and distance in an array
    var minDist = 1e16;
    for (var i = 0; i < arr.length; i++) {
        var coord2 = arr[i];
        var curDist = findDist(coord1, coord2);
        if (curDist < minDist) {

            minDist = curDist;
            var myP = coord2;
        }
    }
    return [myP, minDist];
};

function findDist(coord1, coord2) {
    //Finds distance between two items from coordinates
    if (typeof (coord1) == "number") {
        coord1 = [coord1];
    }
    if (typeof (coord2) == "number") {
        coord2 = [coord2];
    }
    var sumOfSquares = 0;
    for (var i = 0; i < coord1.length; i++) {
        var sum = (coord1[i] - coord2[i]) * (coord1[i] - coord2[i]);
        sumOfSquares += sum;
    }
    return Math.sqrt(sumOfSquares);
};

function findFinalDist(selarr, arr) {
    //Finds min distance from two arrays, push points and distance in an array
    var minDist = 1e16;
    for (var i = 0; i < selarr.length; i++) {
        var coord1 = selarr[i];
        var newDistArr = minstDist(coord1, arr);
        if (newDistArr[1] < minDist) {
            minDist = newDistArr[1];
            var myP1 = coord1;
            var myP2 = newDistArr[0];
        }
    }
    return [myP1, myP2, minDist];
};

function getStartPoint(AB, myItem) {

    var ABrect = AB.artboardRect;
    var abArrX = [ABrect[0], ABrect[2]];
    var abArrY = [ABrect[1], ABrect[3]];

    var itemRect = myItem.geometricBounds;//textFrame.geometricBounds;

    var itemArrX = [itemRect[0], itemRect[2]];
    var itemArrY = [itemRect[1], itemRect[3]];
    var pointX = findFinalDist(itemArrX, abArrX);
    var pointY = findFinalDist(itemArrY, abArrY)


    return { 'coord': [pointX[0], pointY[0]], 'dist': [pointX[2], pointY[2]] };
}

function layerExists(layerName) {
    //Check if layer exist. Return true or false
    try {

        app.activeDocument.layers.getByName(layerName);
        return true;

    } catch (err) {

        return false
    }
};

function activateArtboardWithSelectedItem(MyItem) {

    // Loop through each artboard to find which one contains the selected item
    for (var i = 0; i < doc.artboards.length; i++) {
        doc.artboards.setActiveArtboardIndex(i);
        var artboardBounds = doc.artboards[i].artboardRect; // Format [left, top, right, bottom]

        var itemBounds = MyItem.geometricBounds; // Format [left, top, right, bottom]

        // Check if the selected item is within the current artboard bounds
        if (itemBounds[0] >= artboardBounds[0] && itemBounds[1] <= artboardBounds[1] &&
            itemBounds[2] <= artboardBounds[2] && itemBounds[3] >= artboardBounds[3]) {

            return i; // Return the zero-based index of the artboard
        }
    }

    alert("Item '" + MyItem.contents  + "' is not inside any artboard bounds.");
    return -1; // Return -1 if no artboard contains the selected item
}
